/*
stdint.h
Replacement version for SFU 3.5

Copyright (C) 2008 Jens Arnold

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA
*/ 

#ifndef _STDINT_H_
#define _STDINT_H_

/* exact-width types */

typedef signed char int8_t;
typedef short       int16_t;
typedef int         int32_t;

#define INT8_MAX    0x7f
#define INT8_MIN    (-INT8_MAX - 1)
#define INT16_MAX   0x7fff
#define INT16_MIN   (-INT16_MAX - 1)
#define INT32_MAX   0x7fffffffL
#define INT32_MIN   (-INT32_MAX - 1)

typedef unsigned char  uint8_t;
typedef unsigned short uint16_t;
typedef unsigned int   uint32_t;

#define UINT8_MAX   0xff
#define UINT16_MAX  0xffff
#define UINT32_MAX  0xffffffffL

#ifdef __GNUC__
typedef long long          int64_t;
typedef unsigned long long uint64_t;
#elif _MSC_VER
typedef __int64            int64_t;
typedef unsigned __int64   uint64_t;
#endif

#define INT64_MAX   0x7fffffffffffffffLL
#define INT64_MIN   (-INT64_MAX - 1)
#define UINT64_MAX  0xffffffffffffffffLL

/* minimum-width types */

typedef int8_t   int_least8_t;
typedef int16_t  int_least16_t;
typedef int32_t  int_least32_t;
typedef int64_t  int_least64_t;

#define INT_LEAST8_MAX  INT8_MAX
#define INT_LEAST8_MIN  INT8_MIN
#define INT_LEAST16_MAX INT16_MAX
#define INT_LEAST16_MIN INT16_MIN
#define INT_LEAST32_MAX INT32_MAX
#define INT_LEAST32_MIN INT32_MIN
#define INT_LEAST64_MAX INT64_MAX
#define INT_LEAST64_MIN INT64_MIN

typedef uint8_t  uint_least8_t;
typedef uint16_t uint_least16_t;
typedef uint32_t uint_least32_t;
typedef uint64_t uint_least64_t;

#define UINT_LEAST8_MAX  UINT8_MAX
#define UINT_LEAST16_MAX UINT16_MAX
#define UINT_LEAST32_MAX UINT32_MAX
#define UINT_LEAST64_MAX UINT64_MAX

/* fastest minimum-width types */

typedef int32_t  int_fast8_t;
typedef int32_t  int_fast16_t;
typedef int32_t  int_fast32_t;
typedef int64_t  int_fast64_t;

#define INT_FAST8_MAX  INT32_MAX
#define INT_FAST8_MIN  INT32_MIN
#define INT_FAST16_MAX INT32_MAX
#define INT_FAST16_MIN INT32_MIN
#define INT_FAST32_MAX INT32_MAX
#define INT_FAST32_MIN INT32_MIN
#define INT_FAST64_MAX INT64_MAX
#define INT_FAST64_MIN INT64_MIN

typedef uint32_t uint_fast8_t;
typedef uint32_t uint_fast16_t;
typedef uint32_t uint_fast32_t;
typedef uint64_t uint_fast64_t;

#define UINT_FAST8_MAX  UINT32_MAX
#define UINT_FAST16_MAX UINT32_MAX
#define UINT_FAST32_MAX UINT32_MAX
#define UINT_FAST64_MAX UINT64_MAX

/* greatest-width integer types */

typedef int64_t  intmax_t;
typedef uint64_t uintmax_t;

#define INTMAX_MIN  INT64_MIN
#define INTMAX_MAX  INT64_MAX
#define UINTMAX_MAX UINT64_MAX

/* integer types capable of holding pointers */

#if defined(lp64) || defined(_WIN64)

#ifndef _INTPTR_T_DEFINED
#define _INTPTR_T_DEFINED
typedef int64_t  intptr_t;
typedef uint64_t uintptr_t;
#endif

#define INTPTR_MIN  INT64_MIN
#define INTPTR_MAX  INT64_MAX
#define UINTPTR_MAX UINT64_MAX

#define PTRDIFF_MIN INT64_MIN
#define PTRDIFF_MAX INT64_MAX
#define SIZE_MAX    INT64_MAX

#else /* WIN32 */

#ifndef _INTPTR_T_DEFINED
#define _INTPTR_T_DEFINED
typedef int32_t  intptr_t;
typedef uint32_t uintptr_t;
#endif

#define INTPTR_MIN  INT32_MIN
#define INTPTR_MAX  INT32_MAX
#define UINTPTR_MAX UINT32_MAX

#define PTRDIFF_MIN INT32_MIN
#define PTRDIFF_MAX INT32_MAX
#define SIZE_MAX    INT32_MAX

#endif /* WIN32 */

/* size_t and ssize_t */

#ifndef _SIZE_T_DEFINED
#define _SIZE_T_DEFINED
typedef uintptr_t size_t;
#endif

#ifndef _SSIZE_T_DEFINED
#define _SSIZE_T_DEFINED
typedef intptr_t ssize_t;
#endif

/* some extra limits */

#define SIG_ATOMIC_MIN INT32_MIN
#define SIG_ATOMIC_MAX INT32_MAX

#define WCHAR_MIN      INT32_MIN
#define WCHAR_MAX      INT32_MAX

#define WINT_MIN       INT32_MIN
#define WINT_MAX       INT32_MAX

/* macros for integer constant expressions */

#define INT8_C(c)    c
#define INT16_C(c)   c
#define INT32_C(c)   c ## L
#define INT64_C(c)   c ## LL

#define UINT8_C(c)   c ## U
#define UINT16_C(c)  c ## U
#define UINT32_C(c)  c ## UL
#define UINT64_C(c)  c ## ULL

#define INTMAX_C(c)  c ## LL
#define UINTMAX_C(c) c ## ULL

#endif /* _STDINT_H_ */